#!/bin/bash

# base
BUILD_ARCH=arm64
BUILD_TYPE=Release
PLUGIN_NAME=veRTCFUPlugin
PLUGIN_DIR_NAME=vertc-mac-${BUILD_ARCH}-fu-plugin

# dir
SCRIPT_DIR=`dirname $0`
cd ${SCRIPT_DIR}/..
ROOT_DIR=`pwd`
cd ${ROOT_DIR}

# build
BUILD_DIR=${ROOT_DIR}/build/mac/${BUILD_ARCH}
BINARY_DIR=${BUILD_DIR}/${BUILD_TYPE}
rm -rf ${BUILD_DIR}
mkdir -p ${BUILD_DIR}
cd ${BUILD_DIR}
cmake -DCMAKE_OSX_ARCHITECTURES=${BUILD_ARCH} -DCMAKE_BUILD_TYPE=${BUILD_TYPE} ${ROOT_DIR} -G "Xcode"
wait=`xcodebuild -project ${PLUGIN_NAME}.xcodeproj -arch ${BUILD_ARCH} -configuration ${BUILD_TYPE} GCC_GENERATE_DEBUGGING_SYMBOLS='YES' DEBUG_INFORMATION_FORMAT='dwarf-with-dsym' DWARF_DSYM_FOLDER_PATH=${BINARY_DIR}`
cd ${ROOT_DIR}

# pack
PACK_DIR=${ROOT_DIR}/pack
PLUGIN_DIR=${PACK_DIR}/mac/${BUILD_ARCH}/${PLUGIN_DIR_NAME}
rm -rf ${PLUGIN_DIR}
rm -rf ${PLUGIN_DIR}.zip
mkdir -p ${PLUGIN_DIR}

cp -a ${BINARY_DIR}/* ${PLUGIN_DIR}
#rm -rf ${PLUGIN_DIR}/*.dSYM

cd ${PLUGIN_DIR}
deps=`otool -l libveRTCFUPlugin.dylib`
rpath=0
for line in ${deps}
do
    if [ ${line} == 'path' ]
    then
        rpath=1
        continue
    fi
    
    if [ ${rpath} -eq 1 ]
    then
        echo ${line}
        install_name_tool -delete_rpath ${line} libveRTCFUPlugin.dylib
        rpath=0
    fi
done
install_name_tool -add_rpath @loader_path/ libveRTCFUPlugin.dylib
otool -l libveRTCFUPlugin.dylib
codesign --force --deep --sign - *.dylib
codesign -dvvv libveRTCFUPlugin.dylib

cd ${PLUGIN_DIR}/..
zip -r ${PLUGIN_DIR_NAME}.zip ${PLUGIN_DIR_NAME}
cd ${ROOT_DIR}

#symbol
PDB_DIR=${ROOT_DIR}/symbol
PLUGIN_PDB_DIR=${PDB_DIR}/mac/${BUILD_ARCH}/${PLUGIN_DIR_NAME}-symbol
rm -rf ${PLUGIN_PDB_DIR}/*
mkdir -p ${PLUGIN_PDB_DIR}
cp -a ${BINARY_DIR}/*.dSYM ${PLUGIN_PDB_DIR}
