﻿/*
 *  Copyright (c) 2023 The ByteRtc project authors. All Rights Reserved.
 *  @company ByteDance.Inc
 *  @brief 视频插件接口类
 */

#pragma once

#include <stdint.h>
#if defined(_WIN32)
#define DLL_EXPORTS  __declspec(dllexport)
#else
#define DLL_EXPORTS
#endif

namespace bytertc {
/**
 * @type keytype
 * @brief 自定义视频插件帧
 */
struct VideoPluginFrame {
    int width;            // 视频帧宽，单位：像素
    int height;           // 视频帧高，单位：像素
    int yStride;          // Y数据行的长度
    int uStride;          // U数据行的长度
    int vStride;          // V数据行的长度
    uint8_t* yBuffer;     // Y数据缓冲区
    uint8_t* uBuffer;     // U数据缓冲区
    uint8_t* vBuffer;     // V数据缓冲区
    int rotation;         // 视频帧旋转角度 {0, 90, 180, 270}
    int64_t timestampUs;  // 频帧时间戳，单位：微秒
};

/**
 * @type api
 * @brief 视频插件API
 */
class IVideoPlugin {
public:
    virtual bool init(const char* path) = 0;
    virtual void uninit() = 0;
    virtual bool setParameter(const char* param) = 0;
    virtual const char* getParameter(const char* key) = 0;
    virtual void release() = 0;

public:
    virtual bool processVideoFrame(VideoPluginFrame* videoFrame) = 0;
};
} // namespace bytertc

/**
 * @type type
 * @brief 定义函数类型
 */
typedef bytertc::IVideoPlugin* (*createByteVideoPlugin)();

/**
 * @type api
 * @brief 创建插件
 */
extern "C" DLL_EXPORTS bytertc::IVideoPlugin* createVideoPlugin();