﻿#include "veRTCFUPlugin.h"
#include "common/rapidjson/document.h"
#include "common/rapidjson/writer.h"

#ifdef _WIN32
#include <windows.h>
#else
#include <OpenGL/OpenGL.h>
#endif 

using namespace rapidjson;

#ifdef _WIN32
PIXELFORMATDESCRIPTOR pfd = {
    sizeof(PIXELFORMATDESCRIPTOR),
    1u,
    PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER | PFD_DRAW_TO_WINDOW,
    PFD_TYPE_RGBA,
    32u,
    0u, 0u, 0u, 0u, 0u, 0u,
    8u,
    0u,
    0u,
    0u, 0u, 0u, 0u,
    24u,
    8u,
    0u,
    PFD_MAIN_PLANE,
    0u,
    0u, 0u 
};
HDC hgldc = nullptr;
HGLRC hglrc = nullptr;
#else
CGLContextObj cglContext = nullptr;
#endif 

bool VeRTCFUPlugin::m_envInited = false;

bytertc::IVideoPlugin* createVideoPlugin()
{
    return new VeRTCFUPlugin();
}

bool VeRTCFUPlugin::init(const char* path)
{
    if (m_inited) {
        return true;
    }
    
    // TODO add init code: log and member variables
        
    m_inited = true;
    return true;
}

void VeRTCFUPlugin::uninit()
{
   if (!m_inited) {
       return;
   }
   
   // TODO add uninit code: release memeory
   
   // bundles
    m_loadBundles = false;
    m_updateBundles = false;
    
    m_inited = false;
}

bool VeRTCFUPlugin::setParameter(const char* param)
{
    Document d;
    d.Parse(param);

    if (d.HasParseError()) {
        // LOG_F(ERROR, "invailed json, %s.", param);
        return false;
    }

    // TODO add analyze json string code
    // use load or update bundles
    
    return true; 
}

bool VeRTCFUPlugin::processVideoFrame(bytertc::VideoPluginFrame* videoFrame)
{
    // TODO add process video frame code: A separate thread, 
    // for the first time, initialize opengl, load model, etc.
    
    // 1.init
    if (!initEnv()) {
        return false;
    }

    // 2.make env
    makeCurrent();

    // 3.bundles
    checkBundles();

    // 4.renderer
    rendererFrame(videoFrame);

    // 5.done env
    doneCurrent();

    return true;
}

bool VeRTCFUPlugin::initEnv()
{
    if (m_envInited) {
        return true;
    }
    
    // TODO init 3rd sdk, global init once
       
    if (!initOpenGL()) {
        // LOG_F(ERROR, "initOpenGL failed.");
        return false;
    }

    m_envInited = true;

    return true;
}

bool VeRTCFUPlugin::initOpenGL()
{
#ifdef _WIN32
    HWND hw = CreateWindowExA(
        0, "EDIT", "", ES_READONLY,
        0, 0, 1, 1,
        NULL, NULL,
        GetModuleHandleA(NULL), NULL);
    hgldc = GetDC(hw);
    int spf = ChoosePixelFormat(hgldc, &pfd);
    if (spf == 0) {
        return false;
    }
    int ret = SetPixelFormat(hgldc, spf, &pfd);
    if (!ret) {
        return false;
    }
    hglrc = wglCreateContext(hgldc);
    wglMakeCurrent(hgldc, hglrc);

    // LOG_F(INFO, "initOpenGL, hw=%p hgldc=%p spf=%d ret=%d hglrc=%p", 
        // hw, hgldc, spf, ret, hglrc);
#else
    CGLPixelFormatAttribute attrib[13] = {
        kCGLPFAOpenGLProfile,
        (CGLPixelFormatAttribute)kCGLOGLPVersion_Legacy,
        kCGLPFAAccelerated,
        kCGLPFAColorSize, (CGLPixelFormatAttribute)24,
        kCGLPFAAlphaSize, (CGLPixelFormatAttribute)8,
        kCGLPFADoubleBuffer,
        kCGLPFASampleBuffers, (CGLPixelFormatAttribute)1,
        kCGLPFASamples, (CGLPixelFormatAttribute)4,
        (CGLPixelFormatAttribute)0
    };
    CGLPixelFormatObj pixelFormat = NULL;
    GLint numPixelFormats = 0;
    CGLChoosePixelFormat(attrib, &pixelFormat, &numPixelFormats);
    CGLError err = CGLCreateContext(pixelFormat, NULL, &cglContext);
    if (err) {
        return false;
    }
    // LOG_F(INFO, "initOpenGL, CGLCreateContext.");
#endif

    return true;
}

void VeRTCFUPlugin::makeCurrent()
{
#ifdef _WIN32
    BOOL r = wglMakeCurrent(hgldc, hglrc);
#else
    CGLSetCurrentContext(cglContext);
#endif
}

void VeRTCFUPlugin::doneCurrent()
{
#ifdef _WIN32
    wglMakeCurrent(hgldc, nullptr);
#else
    CGLSetCurrentContext(nullptr);
#endif
}

bool VeRTCFUPlugin::checkBundles()
{
    if (m_loadBundles) {
       // TODO load bundles
       m_loadBundles = true;
    }
    
    if (m_updateBundles) {
       // TODO update bundles
       m_updateBundles = true;
    }

    return true;
}

bool VeRTCFUPlugin::rendererFrame(bytertc::VideoPluginFrame* videoFrame)
{
    if (!m_inited || videoFrame == nullptr) {
        return false;
    }
    
    // TODO use 3rd sdk to renderer video frame

    return true;
}